import { ITEMS_PER_PAGE } from 'constant/others/other';
import CLOUD_STORE_LIST from 'constant/templates/cloud_store/resource';
import useDrupalJsonApi from 'hooks/drupal_jsonapi';
import DataColumn from 'model/DataColumn';
import DataRecord from 'model/DataRecord';
import SortInfo from 'model/SortInfo';
import LoadingSpinner from 'molecules/LoadingSpinner';
import DataTable from 'organisms/DataTable';
import { useContext, useEffect, useState } from 'react';
import { convertEntityDataToDataRecord } from 'service/convert';
import { AppContext } from 'service/state';

/**
 * ListView of data.
 *
 * @returns JSX of CloudStoreTable.
 */
const CloudStoreTable = ({ bundleId, pageIndex }: {
  bundleId: string,
  pageIndex: number,
}) => {

  const { cloudContextList } = useContext(AppContext);
  const { getEntityList } = useDrupalJsonApi();
  const [dataColumnList, setDataColumnList] = useState<DataColumn[]>([]);
  const [dataRecordList, setDataRecordList] = useState<DataRecord[]>([]);
  const [sortInfo, setSortInfo] = useState<SortInfo>({
    key: '', direction: 'ASC'
  });
  const [loadingFlg, setLoadingFlg] = useState(true);

  const template = CLOUD_STORE_LIST.filter((r) => r.bundleId === bundleId)[0];

  /** Update dataColumnList and dataRecordList */
  useEffect(() => {
    const init = async () => {
      // Load launch template's column data.
      const columnList = template.column;
      let newDataColumnList: DataColumn[] = columnList.map((column) => {
        return { key: column.name, label: column.labelName };
      });
      setDataColumnList(newDataColumnList);

      // Create function parameter.
      const parameter = {
        limit: ITEMS_PER_PAGE,
        offset: pageIndex * ITEMS_PER_PAGE,
        filter: {},
        sort: sortInfo
      };

      // Load launch template's data.
      const rawData = await getEntityList('cloud_store', parameter, template.bundleId);
      setDataRecordList(convertEntityDataToDataRecord(template.bundleId, rawData, columnList, cloudContextList, {}));
      setLoadingFlg(false);
    };
    setLoadingFlg(true);
    init();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [cloudContextList, sortInfo, pageIndex]);

  return loadingFlg
    ? <LoadingSpinner />
    : <DataTable
      dataColumnList={dataColumnList}
      dataRecordList={dataRecordList}
      sortInfo={sortInfo}
      setSortInfo={setSortInfo}
      hasOperationLinks={true}
      operationLinksName="Operations" />;
}

export default CloudStoreTable;
