<?php

namespace Drupal\openstack\Entity;

use Drupal\cloud\Entity\CloudContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the OpenStack Quota entity.
 *
 * @ingroup openstack
 *
 * @ContentEntityType(
 *   id = "openstack_quota",
 *   id_plural = "openstack_quotas",
 *   label = @Translation("Quota"),
 *   label_collection = @Translation("Quotas"),
 *   label_singular = @Translation("Quota"),
 *   label_plural = @Translation("Quotas"),
 *   handlers = {
 *     "view_builder" = "Drupal\openstack\Entity\OpenStackQuotaViewBuilder",
 *     "list_builder" = "Drupal\cloud\Controller\CloudContentListBuilder",
 *     "views_data"   = "Drupal\openstack\Entity\OpenStackQuotaViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\openstack\Form\OpenStackQuotaEditForm",
 *       "edit" = "Drupal\openstack\Form\OpenStackQuotaEditForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\openstack\Controller\OpenStackQuotaAccessControlHandler",
 *   },
 *   base_table = "openstack_quota",
 *   admin_permission = "administer openstack quota",
 *   fieldable = TRUE,
 *   entity_keys = {
 *     "id"    = "id",
 *     "label" = "name",
 *     "uuid"  = "uuid",
 *   },
 *   links = {
 *     "canonical" = "/clouds/openstack/{cloud_context}/quota/{openstack_quota}",
 *     "edit-form" = "/clouds/openstack/{cloud_context}/quota/{openstack_quota}/edit",
 *     "collection" = "/clouds/openstack/{cloud_context}/quota",
 *   },
 *   field_ui_base_route = "openstack_quota.settings"
 * )
 */
class OpenStackQuota extends CloudContentEntityBase implements OpenStackQuotaInterface {

  /**
   * {@inheritdoc}
   */
  public function getName(): ?string {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name = ''): OpenStackQuotaInterface {
    return $this->set('name', $name);
  }

  /**
   * {@inheritdoc}
   */
  public function getProjectId(): ?string {
    return $this->get('project_id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setProjectId($project_id = ''): OpenStackQuotaInterface {
    return $this->set('project_id', $project_id);
  }

  /**
   * {@inheritdoc}
   */
  public function getInstances(): ?int {
    return $this->get('instances')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setInstances($instances): OpenStackQuotaInterface {
    return $this->set('instances', $instances);
  }

  /**
   * {@inheritdoc}
   */
  public function getCores(): ?int {
    return $this->get('cores')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCores($cores): OpenStackQuotaInterface {
    return $this->set('cores', $cores);
  }

  /**
   * {@inheritdoc}
   */
  public function getRam(): ?int {
    return $this->get('ram')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setRam($ram): OpenStackQuotaInterface {
    return $this->set('ram', $ram);
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataItems(): ?int {
    return $this->get('metadata_items')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setMetadataItems($metadata_items): OpenStackQuotaInterface {
    return $this->set('metadata_items', $metadata_items);
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyPairs(): ?int {
    return $this->get('key_pairs')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setKeyPairs($key_pairs): OpenStackQuotaInterface {
    return $this->set('key_pairs', $key_pairs);
  }

  /**
   * {@inheritdoc}
   */
  public function getServerGroups(): ?int {
    return $this->get('server_groups')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setServerGroups($server_groups): OpenStackQuotaInterface {
    return $this->set('server_groups', $server_groups);
  }

  /**
   * {@inheritdoc}
   */
  public function getServerGroupMembers(): ?int {
    return $this->get('server_group_members')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setServerGroupMembers($server_group_members): OpenStackQuotaInterface {
    return $this->set('server_group_members', $server_group_members);
  }

  /**
   * {@inheritdoc}
   */
  public function getInjectedFiles(): ?int {
    return $this->get('injected_files')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setInjectedFiles($injected_files): OpenStackQuotaInterface {
    return $this->set('injected_files', $injected_files);
  }

  /**
   * {@inheritdoc}
   */
  public function getInjectedFileContentBytes(): ?int {
    return $this->get('injected_file_content_bytes')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setInjectedFileContentBytes($injected_file_content_bytes): OpenStackQuotaInterface {
    return $this->set('injected_file_content_bytes', $injected_file_content_bytes);
  }

  /**
   * {@inheritdoc}
   */
  public function getInjectedFilePathBytes(): ?int {
    return $this->get('injected_file_path_bytes')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setInjectedFilePathBytes($injected_file_path_bytes): OpenStackQuotaInterface {
    return $this->set('injected_file_path_bytes', $injected_file_path_bytes);
  }

  /**
   * {@inheritdoc}
   */
  public function getVolumes(): ?int {
    return $this->get('volumes')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setVolumes($volumes): OpenStackQuotaInterface {
    return $this->set('volumes', $volumes);
  }

  /**
   * {@inheritdoc}
   */
  public function getSnapshots(): ?int {
    return $this->get('snapshots')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSnapshots($snapshots): OpenStackQuotaInterface {
    return $this->set('snapshots', $snapshots);
  }

  /**
   * {@inheritdoc}
   */
  public function getGigaBytes(): ?int {
    return $this->get('gigabytes')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setGigaBytes($gigabytes): OpenStackQuotaInterface {
    return $this->set('gigabytes', $gigabytes);
  }

  /**
   * {@inheritdoc}
   */
  public function getNetwork(): ?int {
    return $this->get('network')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setNetwork($network): OpenStackQuotaInterface {
    return $this->set('network', $network);
  }

  /**
   * {@inheritdoc}
   */
  public function getSubnet(): ?int {
    return $this->get('subnet')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSubnet($subnet): OpenStackQuotaInterface {
    return $this->set('subnet', $subnet);
  }

  /**
   * {@inheritdoc}
   */
  public function getPort(): ?int {
    return $this->get('port')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setPort($port): OpenStackQuotaInterface {
    return $this->set('port', $port);
  }

  /**
   * {@inheritdoc}
   */
  public function getRouter(): ?int {
    return $this->get('router')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setRouter($router): OpenStackQuotaInterface {
    return $this->set('router', $router);
  }

  /**
   * {@inheritdoc}
   */
  public function getFloatingIp(): ?int {
    return $this->get('floatingip')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setFloatingIp($floating_ip): OpenStackQuotaInterface {
    return $this->set('floatingip', $floating_ip);
  }

  /**
   * {@inheritdoc}
   */
  public function getSecurityGroup(): ?int {
    return $this->get('security_group')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSecurityGroup($security_group): OpenStackQuotaInterface {
    return $this->set('security_group', $security_group);
  }

  /**
   * {@inheritdoc}
   */
  public function getSecurityGroupRule(): ?int {
    return $this->get('security_group_rule')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSecurityGroupRule($security_group_rule): OpenStackQuotaInterface {
    return $this->set('security_group_rule', $security_group_rule);
  }

  /**
   * {@inheritdoc}
   */
  public function getRefreshed(): int {
    return $this->get('refreshed')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setRefreshed($time): OpenStackQuotaInterface {
    return $this->set('refreshed', $time);
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {

    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setDescription(t('The ID of the Quota entity.'))
      ->setReadOnly(TRUE);

    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('The UUID of the Quota entity.'))
      ->setReadOnly(TRUE);

    $fields['cloud_context'] = BaseFieldDefinition::create('string')
      ->setRequired(TRUE)
      ->setLabel(t('Cloud service provider ID'))
      ->setDescription(t('A unique ID for the cloud service provider.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ]);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of quota.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setReadOnly(TRUE);

    $fields['project_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Project ID'))
      ->setDescription(t('The ID of project.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setReadOnly(TRUE);

    $fields['instances'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Instances'))
      ->setDescription(t('The number of allowed servers for each tenant.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['cores'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('VCPUs'))
      ->setDescription(t('The number of allowed server cores for each tenant.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['ram'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('RAM(MiB)'))
      ->setDescription(t('The amount of allowed server RAM, in MiB, for each tenant.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['metadata_items'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Metadata Items'))
      ->setDescription(t('The number of allowed metadata items for each server.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['key_pairs'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Key Pairs'))
      ->setDescription(t('The number of allowed key pairs for each user.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['server_groups'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Server Groups'))
      ->setDescription(t('The number of allowed server groups for each tenant.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['server_group_members'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Server Group Members'))
      ->setDescription(t('The number of allowed members for each server group.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['injected_files'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Injected Files'))
      ->setDescription(t('The number of allowed injected files for each tenant.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['injected_file_content_bytes'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Injected File Content(Bytes)'))
      ->setDescription(t('The number of allowed bytes for each injected file path.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['injected_file_path_bytes'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Length of Injected File Path'))
      ->setDescription(t('The number of allowed bytes for each injected file path.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['volumes'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Volumes'))
      ->setDescription(t('The number of volumes that are allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['snapshots'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Volume snapshots'))
      ->setDescription(t('The number of snapshots that are allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['gigabytes'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Total Size of volumes and snapshots (GiB)'))
      ->setDescription(t('The size (GB) of volumes and snapshots that are allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['network'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Networks'))
      ->setDescription(t('The number of networks allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['subnet'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Subnets'))
      ->setDescription(t('The number of subnets allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['port'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Ports'))
      ->setDescription(t('The number of ports allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['router'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Routers'))
      ->setDescription(t('The number of routers allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['floatingip'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Floating IPs'))
      ->setDescription(t('The number of floating IP addresses allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['security_group'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Security Groups'))
      ->setDescription(t('The number of security groups allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['security_group_rule'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Security Group Rules'))
      ->setDescription(t('The number of security group rules allowed for each project.'))
      ->setSetting('min', -1)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
        'weight' => -5,
      ]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('When the entity was created.'))
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'timestamp',
        'weight' => -5,
        'settings' => [
          'date_format' => 'short',
        ],
      ]);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['refreshed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Refreshed'))
      ->setDescription(t('The time that the entity was last refreshed.'));

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('Enter the user ID of the Quota entity author.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback('Drupal\node\Entity\Node::getCurrentUserId')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'match_limit' => 10,
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * Get VNIC type allowed values.
   *
   * @return array
   *   The VNIC type allowed values.
   */
  public static function getVnicTypeAllowedValues(): array {
    return [
      'normal' => t('Normal'),
      'direct' => t('Direct'),
      'direct-physical' => t('Direct Physical'),
      'macvtap' => t('MacVTap'),
      'baremetal' => t('Bare Metal'),
      'virtio-forwarder' => t('Virtio Forwarder'),
    ];
  }

}
