<?php

namespace Drupal\openstack\Service;

use Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface;
use Drupal\aws_cloud\Entity\Ec2\ImageInterface;
use Drupal\aws_cloud\Entity\Ec2\InstanceInterface;
use Drupal\aws_cloud\Entity\Ec2\KeyPairInterface;
use Drupal\aws_cloud\Entity\Ec2\NetworkInterfaceInterface;
use Drupal\aws_cloud\Entity\Ec2\SecurityGroupInterface;
use Drupal\aws_cloud\Entity\Ec2\SnapshotInterface;
use Drupal\aws_cloud\Entity\Ec2\VolumeInterface;
use Drupal\cloud\Entity\CloudLaunchTemplateInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\openstack\Entity\OpenStackNetworkExInterface;
use Drupal\openstack\Entity\OpenStackPortInterface;
use Drupal\openstack\Entity\OpenStackQuotaInterface;
use Drupal\openstack\Entity\OpenStackRouterInterface;
use Drupal\openstack\Entity\OpenStackStackInterface;
use Drupal\openstack\Entity\OpenStackSubnetInterface;

/**
 * Provides OpenStackOperationsService interface.
 */
interface OpenStackOperationsServiceInterface {

  /**
   * Delete an OpenStack Instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackInstance(InstanceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack image.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ImageInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackImage(ImageInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Start an OpenStack Instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The OpenStack instance entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function startOpenStackInstance(InstanceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Stop an OpenStack Instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The OpenStack instance entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function stopOpenStackInstance(InstanceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Reboot an OpenStack Instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The OpenStack instance entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function rebootOpenStackInstance(InstanceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack image.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ImageInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackImage(ImageInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack image.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ImageInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackImage(ImageInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack KeyPair.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\KeyPairInterface $entity
   *   The OpenStack key pair entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackKeyPair(KeyPairInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack KeyPair.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\KeyPairInterface $entity
   *   The OpenStack key pair entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackKeyPair(KeyPairInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack KeyPair.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\KeyPairInterface $entity
   *   The OpenStack key pair entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackKeyPair(KeyPairInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack NetworkInterface.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\NetworkInterfaceInterface $entity
   *   The OpenStack network interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackNetworkInterface(NetworkInterfaceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack Network.
   *
   * @param \Drupal\openstack\Entity\OpenStackNetworkExInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackNetwork(OpenStackNetworkExInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack Subnet.
   *
   * @param \Drupal\openstack\Entity\OpenStackSubnetInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackSubnet(OpenStackSubnetInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack Port.
   *
   * @param \Drupal\openstack\Entity\OpenStackPortInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackPort(OpenStackPortInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack Router.
   *
   * @param \Drupal\openstack\Entity\OpenStackRouterInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackRouter(OpenStackRouterInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Add an interface to OpenStack Router.
   *
   * @param \Drupal\openstack\Entity\OpenStackRouterInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function addOpenStackRouterInterface(OpenStackRouterInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Remove an interface to OpenStack Router.
   *
   * @param \Drupal\openstack\Entity\OpenStackRouterInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function removeOpenStackRouterInterface(OpenStackRouterInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack Instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\InstanceInterface $entity
   *   The instance entity.
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackInstance(InstanceInterface $entity, array $form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack NetworkInterface.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\NetworkInterfaceInterface $entity
   *   The OpenStack network interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackNetworkInterface(NetworkInterfaceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack NetworkInterface.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\NetworkInterfaceInterface $entity
   *   The OpenStack network interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackNetworkInterface(NetworkInterfaceInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack Network.
   *
   * @param \Drupal\openstack\Entity\OpenStackNetworkExInterface $entity
   *   The OpenStack network entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackNetwork(OpenStackNetworkExInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack Subnet.
   *
   * @param \Drupal\openstack\Entity\OpenStackSubnetInterface $entity
   *   The OpenStack subnet entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackSubnet(OpenStackSubnetInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack Port.
   *
   * @param \Drupal\openstack\Entity\OpenStackPortInterface $entity
   *   The OpenStack port entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackPort(OpenStackPortInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack Router.
   *
   * @param \Drupal\openstack\Entity\OpenStackRouterInterface $entity
   *   The OpenStack router entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackRouter(OpenStackRouterInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack Quota.
   *
   * @param \Drupal\openstack\Entity\OpenStackQuotaInterface $entity
   *   The OpenStack quota entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackQuota(OpenStackQuotaInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack Network.
   *
   * @param \Drupal\openstack\Entity\OpenStackNetworkExInterface $entity
   *   The OpenStack network interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackNetwork(OpenStackNetworkExInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack Subnet.
   *
   * @param \Drupal\openstack\Entity\OpenStackSubnetInterface $entity
   *   The OpenStack subnet interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackSubnet(OpenStackSubnetInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack Port.
   *
   * @param \Drupal\openstack\Entity\OpenStackPortInterface $entity
   *   The OpenStack port interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackPort(OpenStackPortInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack Router.
   *
   * @param \Drupal\openstack\Entity\OpenStackRouterInterface $entity
   *   The OpenStack router interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackRouter(OpenStackRouterInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack Stack.
   *
   * @param \Drupal\openstack\Entity\OpenStackStackInterface $entity
   *   The OpenStack stack interface entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackStack(OpenStackStackInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack floating IP.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity
   *   The OpenStack floating IP entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackFloatingIp(ElasticIpInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack floating IP.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackFloatingIp(ElasticIpInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack floating IP.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackFloatingIp(ElasticIpInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Disassociate an OpenStack floating IP.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function disassociateOpenStackFloatingIp(ElasticIpInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack SecurityGroup.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SecurityGroupInterface $entity
   *   The OpenStack security group entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackSecurityGroup(SecurityGroupInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack SecurityGroup.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SecurityGroupInterface $entity
   *   The OpenStack security group entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackSecurityGroup(SecurityGroupInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Revoke an OpenStack SecurityGroup.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SecurityGroupInterface $entity
   *   The OpenStack security group entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function revokeOpenStackSecurityGroup(SecurityGroupInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack SecurityGroup.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SecurityGroupInterface $entity
   *   The OpenStack security group entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackSecurityGroup(SecurityGroupInterface $entity, array $form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack snapshot.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SnapshotInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackSnapshot(SnapshotInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack snapshot.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SnapshotInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackSnapshot(SnapshotInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Associate an OpenStack Floating IP.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity
   *   The OpenStack Floating IP entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function associateOpenStackFloatingIp(ElasticIpInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Helper function that loads all the private IPs for an instance.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\ElasticIpInterface $entity
   *   The OpenStack Floating IP entity.
   * @param int $instance_id
   *   The instance ID.
   *
   * @return array
   *   An array of IP addresses.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getPrivateIps(ElasticIpInterface $entity, $instance_id): array;

  /**
   * Edit an OpenStack snapshot.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\SnapshotInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackSnapshot(SnapshotInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Create an OpenStack volume.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\VolumeInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function createOpenStackVolume(VolumeInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Delete an OpenStack volume.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\VolumeInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function deleteOpenStackVolume(VolumeInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack volume.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\VolumeInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackVolume(VolumeInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Import an OpenStack KeyPair.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\KeyPairInterface $entity
   *   The openStack key pair entity.
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return bool
   *   TRUE when the process succeeds.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function importOpenStackKeyPair(KeyPairInterface $entity, array $form, FormStateInterface $form_state): bool;

  /**
   * Attach an OpenStack volume.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\VolumeInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function attachOpenStackVolume(VolumeInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Detach an OpenStack volume.
   *
   * @param \Drupal\aws_cloud\Entity\Ec2\VolumeInterface $entity
   *   The OpenStack image entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function detachOpenStackVolume(VolumeInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Copy an OpenStack CloudLaunchTemplate.
   *
   * @param \Drupal\cloud\Entity\CloudLaunchTemplateInterface $entity
   *   The CloudLaunchTemplate entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function copyOpenStackCloudLaunchTemplate(CloudLaunchTemplateInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Edit an OpenStack CloudLaunchTemplate.
   *
   * @param \Drupal\cloud\Entity\CloudLaunchTemplateInterface $entity
   *   The CloudLaunchTemplate entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function editOpenStackCloudLaunchTemplate(CloudLaunchTemplateInterface $entity, array &$form, FormStateInterface $form_state): bool;

  /**
   * Review OpenStack CloudLaunchTemplate.
   *
   * @param \Drupal\cloud\Entity\CloudLaunchTemplateInterface $entity
   *   The CloudLaunchTemplate entity.
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE when the process succeeds.
   */
  public function reviewOpenStackCloudLaunchTemplate(CloudLaunchTemplateInterface $entity, array &$form, FormStateInterface $form_state): bool;

}
