@minimal @ci_job
Feature: Launch, stop, and terminate an instance for OpenStack as "Authenticated User"

  @api
  Scenario Outline: Create a launch template by a user and a cloud admin.
    Given I am logged in as user "<user>"
    When I visit "/clouds/design/server_template/{{ cloud_context }}/openstack/add"
    And I should see the heading "Add OpenStack"
    And I enter "<instance_name>" for "Name"
    And I select "{{ instance_flavor }}" from "Flavor"
    And I select "{{ image_name }}" from "Image ID"
    And I select "{{ availability_zone }}" from "Availability zone"
    And I select "{{ security_group_name }}" from "Security groups"
    And I select "{{ key_pair_name }}" from "SSH key"
    # Make sure we have this network interface at OpenStack Horizon dashboard.
    And I select "{{ network_name }}" from "Network interface"
    And I press "Save"
    Then the url should match "/clouds/design/server_template/{{ cloud_context }}/"
    And I should see the success message "has been created"
    And I should see neither error nor warning messages
    And I should see the heading "<instance_name>"
    Examples:
      | user                          | instance_name                 |
      | {{ drupal_user_name }}        | {{ instance_name_operate_1 }} |
      | {{ drupal_cloud_admin_name }} | {{ instance_name_operate_2 }} |

  @api
  Scenario Outline: Launch an instance by a user
    Given I am logged in as user "{{ drupal_user_name }}"
    When I visit "/clouds/design/server_template/{{ cloud_context }}"
    And I click "<instance_name>"
    And I should see the heading "<instance_name>"
    And I click "Launch"
    And the url should match "/launch"
    And I press "Launch"
    Then the url should match "/clouds/openstack/{{ cloud_context }}/instance"
    And I should see the success message "has been launched"
    And I should see neither error nor warning messages
    And I wait {{ wait }} milliseconds
    And I click "Refresh"
    And I should see "{{ instance_flavor }}" in the "<instance_name>" row
    And I should see "{{ availability_zone }}" in the "<instance_name>" row
    Examples:
      | instance_name                 |
      | {{ instance_name_operate_1 }} |
      | {{ instance_name_operate_2 }} |

  @api
  Scenario Outline: View the created instance and its author
    Given I am logged in as user "{{ drupal_user_name }}"
    When I visit "/clouds/openstack/{{ cloud_context }}/instance"
    And I should see the link "<instance_name>"
    And I click "<instance_name>"
    Then the url should match "/clouds/openstack/{{ cloud_context }}/instance/"
    And I should see "<instance_name>" in the "page_header"
    And I should see "{{ instance_flavor }}" in the "Instance type"
    And I should not see "{{ instance_flavor_as_first_option }}"
    And I should see "{{ image_name }}" in the "Image"
    And I should see "{{ availability_zone }}" in the "Availability Zone"
    And I should see "{{ security_group_name }}" in the "Security groups"
    And I should see "{{ key_pair_name }}" in the "Key pair name"
    # The author should be the launcher, not the author of the launch template.
    And I should see "{{ drupal_user_name }}" in the "Authored by"
    And I should see neither error nor warning messages
    Examples:
      | instance_name                 |
      | {{ instance_name_operate_1 }} |
      | {{ instance_name_operate_2 }} |

  @api
  Scenario Outline: Stop the running instance
    Given I am logged in as user "{{ drupal_user_name }}"
    When I visit "/clouds/openstack/{{ cloud_context }}/instance"
    And I click "Refresh"
    And I should see "running" in the "<instance_name>" row
    And I wait {{ wait }} milliseconds
    And I check the box in the "<instance_name>" row
    And I select "Stop Instance(s)" from "Action"
    And I press "Apply to selected items"
    Then the url should match "/clouds/openstack/{{ cloud_context }}/instance/stop_multiple"
    And I should see "<instance_name>"
    And I press "Stop"
    And I should be on "/clouds/openstack/{{ cloud_context }}/instance"
    And I should see the success message "has been stopped"
    And I should see neither error nor warning messages
    And I click "Refresh"
    And I should see "stopped" in the "<instance_name>" row
    Examples:
      | instance_name                 |
      | {{ instance_name_operate_1 }} |
      | {{ instance_name_operate_2 }} |

  @api
  Scenario Outline: Terminate the stopped instance
    Given I am logged in as user "{{ drupal_user_name }}"
    When I visit "/clouds/openstack/{{ cloud_context }}/instance"
    And I click "Refresh"
    And I should see "stopped" in the "<instance_name>" row
    And I should see the link "<instance_name>"
    And I wait {{ wait }} milliseconds
    And I click "<instance_name>"
    And the url should match "/clouds/openstack/{{ cloud_context }}/instance/"
    And I wait {{ wait }} milliseconds
    And I click "Delete"
    Then the url should match "/terminate"
    And I press "Delete | Terminate"
    And I should be on "/clouds/openstack/{{ cloud_context }}/instance"
    And I should see the success message "has been deleted"
    And I should see neither error nor warning messages
    And I wait {{ wait_deleted }} milliseconds
    And I click "Refresh"
    And I should see with "stopped" in the "<instance_name>" row, no rows, or no table
    Examples:
      | instance_name                 |
      | {{ instance_name_operate_1 }} |
      | {{ instance_name_operate_2 }} |

  @api @javascript
  Scenario Outline: Delete the created launch template by the authored user
    Given I am logged in as user "<user>"
    When I visit "/clouds/design/server_template/{{ cloud_context }}"
    And I click "Refresh"
    And I should see the link "<instance_name>"
    And I wait {{ wait }} milliseconds
    And I click "<instance_name>"
    And the url should match "/clouds/design/server_template/{{ cloud_context }}/"
    And I click "Delete"
    Then the url should match "/delete"
    And I press "Delete"
    And I wait for AJAX to finish
    And I should be on "/clouds/design/server_template/{{ cloud_context }}"
    And I should see the success message "has been deleted"
    And I should see neither error nor warning messages
    And I wait {{ wait_deleted }} milliseconds
    And I click "Refresh"
    And I should not see the link "<instance_name>"
    Examples:
      | user                          | instance_name                 |
      | {{ drupal_user_name }}        | {{ instance_name_operate_1 }} |
      | {{ drupal_cloud_admin_name }} | {{ instance_name_operate_2 }} |
